unit DFInitLng;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IniFiles, ComCtrls, ExtCtrls, StdCtrls, Buttons, Menus;

type
  TLgnWriteMode = (DFLng_Append, DFLngCreate);

  TDFInitLng = class(TComponent)
  private
    ini: TIniFile;
    FWriteLog: boolean;
    FFileName: string;
    FItemSeperator: string;
    FMessageHeader: string;
    procedure ReplaceLF(var value: string);
  public
    constructor Create(AOwner: TComponent); override;
    procedure WriteComponentProperties(LgnWriteMode: TLgnWriteMode);
    procedure WriteInternalStrings(var InternalStrings: array of string;
                                   Offset: Integer; LgnWriteMode: TLgnWriteMode);
  published
    property WriteLog: boolean read FWriteLog write FWriteLog default false;
    property FileName: string read FFileName write FFileName;
    property ItemSeperator: string read FItemSeperator write FItemSeperator;
    property MessageHeader: string read FMessageHeader write FMessageHeader;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DFrisch', [TDFInitLng]);
end;


{ TDFInitLng }

// Replace #13#10 with ; for storing Items in a single line
constructor TDFInitLng.Create(AOwner: TComponent);
begin
  inherited;
  FWriteLog := false;
  FMessageHeader := 'Messages';
end;

procedure TDFInitLng.ReplaceLF(var value: string);
var
  i: Integer;
  vPos: Integer;
begin
  vPos := Pos(#13#10, value);
  while vPos > 0 do
  begin
    Delete(value, vPos, 2);
    Insert(FItemSeperator, value, vPos);
    vPos := Pos(#13#10, value);
  end;
  if value <> '' then
    if value[Length(value)] = ';' then
      Delete(value, Length(value), 1);
end;

procedure TDFInitLng.WriteComponentProperties(LgnWriteMode: TLgnWriteMode);
var
  FormsIdx, j, k: Integer;
  FormName: string;
  C: TComponent;
  C2: TComponent;
  ItemCount: Integer;
  ItemString: string;
begin
  if LgnWriteMode = DFLngCreate then
    if FileExists(FFileName) then
      DeleteFile(FFileName);

  ini := TIniFile.Create(FFileName);

  try
    for FormsIdx := 0 to Application.ComponentCount - 1 do
    begin
      if Application.Components[FormsIdx] is TForm then
      begin
        C2 := Application.Components[FormsIdx];
        FormName := C2.Name;
        ini.WriteString(FormName, FormName, (C2 as TForm).Caption);

        for j := 0 to Application.Components[FormsIdx].ComponentCount - 1 do
        begin
          c := Application.Components[FormsIdx].Components[j];
          // be carefully with ClassNames - if you compare with "=" they are case sensitive!
          if c.ClassName = 'TButton' then
          begin
            ini.WriteString(FormName, c.Name, (c as TButton).Caption);
            if (c as TButton).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TButton).Hint);
            continue;
          end;
          if c.ClassName = 'TLabel' then
          begin
            ini.WriteString(FormName, c.Name, (c as TLabel).Caption);
            if (c as TLabel).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TLabel).Hint);
            continue;
          end;
          if c.ClassName = 'TComboBox' then
          begin
            ItemCount := (c as TComboBox).Items.Count;
            ItemString := (c as TComboBox).Items.Text;
            ReplaceLF(ItemString);
            ini.WriteString(FormName, c.Name + 'Items', (c as TComboBox).Items.Text);
            ini.WriteString(FormName, c.Name + 'Items', ItemString);
            if (c as TComboBox).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TComboBox).Hint);
            continue;
          end;
          if c.ClassName = 'TOpenDialog' then
          begin
            ini.WriteString(FormName, c.Name + 'Title', (c as TOpenDialog).Title);
            ini.WriteString(FormName, c.Name + 'Filter', (c as TOpenDialog).Filter);
            continue;
          end;
          if c.ClassName = 'TSaveDialog' then
          begin
            ini.WriteString(FormName, c.Name + 'Title', (c as TSaveDialog).Title);
            ini.WriteString(FormName, c.Name + 'Filter', (c as TSaveDialog).Filter);
            continue;
          end;
          if c.ClassName = 'TSpeedButton' then
          begin
            ini.WriteString(FormName, c.Name, (c as TSpeedButton).Caption);
            if (c as TSpeedButton).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TSpeedButton).Hint);
            continue;
          end;
          if c.ClassName = 'TToolButton' then
          begin
            ini.WriteString(FormName, c.Name, (c as TToolButton).Caption);
            if (c as TToolButton).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TToolButton).Hint);
            continue;
          end;
          if c.ClassName = 'TCheckBox' then
          begin
            ini.WriteString(FormName, c.Name, (c as TCheckBox).Caption);
            if (c as TCheckBox).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TCheckBox).Hint);
            continue;
          end;
          if c.ClassName = 'TRadioButton' then
          begin
            ini.WriteString(FormName, c.Name, (c as TRadiobutton).Caption);
            if (c as TRadioButton).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TRadiobutton).Hint);
            continue;
          end;
          if c.ClassName = 'TBitBtn' then
          begin
            ini.WriteString(FormName, c.Name, (c as TBitBtn).Caption);
            if (c as TBitBtn).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TBitBtn).Hint);
            continue;
          end;
          if c.ClassName = 'TMenuItem' then
          begin
            ini.WriteString(FormName, c.Name, (c as TMenuItem).Caption);
            if (c as TMenuItem).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TMenuItem).Hint);
            continue;
          end;
          if c.ClassName = 'TListBox' then
          begin
            ItemCount := (c as TListBox).Items.Count;
            ItemString := (c as TListBox).Items.Text;
            ReplaceLF(ItemString);
            ini.WriteString(FormName, c.Name + 'Items', ItemString);
            if (c as TListBox).Hint <> '' then
              ini.WriteString(FormName, c.Name + 'Hint', (c as TListBox).Hint);
            continue;
          end;
          if WriteLog then
            ini.WriteString(FormName, c.ClassName, 'LOG: Component not parsed yet');
        end;
      end;
    end;
  finally
    ini.Free;
  end;
end;

procedure TDFInitLng.WriteInternalStrings(var InternalStrings: array of string;
                                          Offset: Integer;
                                          LgnWriteMode: TLgnWriteMode);
var
  InternalStringIdx: Integer;
begin
  if LgnWriteMode = DFLngCreate then
    if FileExists(FFileName) then
      DeleteFile(FFileName);

  ini := TIniFile.Create(FFileName);
  try
    ini.EraseSection(FMessageHeader);
    for InternalStringIdx := 0 to Length(InternalStrings) - 1 do
      ini.WriteString(FMessageHeader, IntToStr(Offset + InternalStringIdx),
                      InternalStrings[InternalStringIdx]);
  finally
    ini.Free;
  end;
end;

end.
