unit DFUseLng;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry,
  StdCtrls, FileCtrl, IniFiles, Buttons, ComCtrls, Menus;

type
  TDFUseLng = class(TComponent)
  private
    { Private-Deklarationen }
    FLanguageFilesDirectory: string;
    FLanguageFile: string; // wenn '' dann default
    procedure SetLanguageFilesDirectory(value: string);
    procedure ReplaceLF(var value: string; ReplaceWith: string);
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    procedure GetLanguages(values: TStrings);
    procedure FillMsgArray(IniFileName: string; var s: array of string; StartID: Integer);
    function  GetActiveLanguage(RootKey: dword; Key: string; value: string): string;
    function  SetActiveLanguage(RootKey: dword; Key: string; value: string; Lan: string): boolean;
    procedure InitForm(IniFileName: string; Frm: TForm);
  published
    { Published-Deklarationen }
    property LanguageFilesDirectory: string read FLanguageFilesDirectory write SetLanguageFilesDirectory;
    property LanguageFile: string read FLanguageFile write FLanguageFile;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('DFrisch', [TDFUseLng]);
end;

{ TLgnSupport }

procedure TDFUseLng.FillMsgArray(IniFileName: string; var s: array of string; StartID: Integer);
var
  ini: TIniFile;
  Idx: Integer;
begin
  if FileExists(IniFileName) then
  begin
    ini := TIniFile.Create(IniFileName);
    try
      for Idx := 0 to Length(s) - 1 do
      begin
        s[Idx] := ini.ReadString('OtherStrings', IntToStr(Idx + StartID), s[Idx]);
      end;
    finally
      ini.Free;
    end;
  end;
end;

function TDFUseLng.GetActiveLanguage(RootKey: dword; Key,
  value: string): string;
var
  vReg: TRegistry;
begin
  Result := '';
  vReg := TRegistry.Create;
  try
    vReg.RootKey := RootKey;

    if vReg.KeyExists(Key) then
      if vReg.OpenKey(Key, false) then
      begin
        if vReg.ValueExists(value) then
          Result := vReg.ReadString(value);
        vReg.CloseKey();
      end;
  finally
    vReg.Free;
  end;
end;

procedure TDFUseLng.GetLanguages(values: TStrings);
var
  SrcRec: TSearchRec;
  found: boolean;
begin
  found := false;
  values.Clear;
  if DirectoryExists(FLanguageFilesDirectory) then
  begin
    if FindFirst(FLanguageFilesDirectory + '\*.lng', faAnyFile, SrcRec) = 0 then
    repeat
      found := true;
      values.Add(Copy(SrcRec.Name, 1, Length(SrcRec.Name) - 4));
    until FindNext(SrcRec) <> 0;

    if found then
      FindClose(SrcRec);
  end;
end;

procedure TDFUseLng.InitForm(IniFileName: string; Frm: TForm);
var
  i: Integer;
  ini: TIniFile;
  FormName: string;
  C: TComponent;
  ItemCount: Integer;
  ItemString: string;
  s: string;
begin
  if FileExists(IniFileName) then
  begin
    ini := TIniFile.Create(IniFileName);

    try
      FormName := Frm.Name;
      frm.Caption := ini.ReadString(FormName, FormName, frm.Caption);
      FormName := Frm.Name;
      for i := 0 to frm.ComponentCount - 1 do
      begin
        c := frm.Components[i];
        if c.ClassName = 'TButton' then
        begin
          (C as TButton).Caption := ini.ReadString(FormName, (C as TButton).Name, (C as TButton).Caption);
          (C as TButton).Hint := ini.ReadString(FormName, (C as TButton).Name + 'Hint', (C as TButton).Hint);
          continue;
        end;
        if c.ClassName = 'TLabel' then
        begin
          (C as TLabel).Caption := ini.ReadString(FormName, (C as TLabel).Name, (C as TLabel).Caption);
          (C as TLabel).Hint := ini.ReadString(FormName, (C as TLabel).Name + 'Hint', (C as TLabel).Hint);
          continue;
        end;
        if c.ClassName = 'TCheckBox' then
        begin
          (C as TCheckBox).Caption := ini.ReadString(FormName, (C as TCheckBox).Name, (C as TCheckBox).Caption);
          (C as TCheckBox).Hint := ini.ReadString(FormName, (C as TCheckBox).Name + 'Hint', (C as TCheckBox).Hint);
          continue;
        end;
        if c.ClassName = 'TBitBtn' then
        begin
          (C as TBitBtn).Caption := ini.ReadString(FormName, (C as TBitBtn).Name, (C as TBitBtn).Caption);
          (C as TBitBtn).Hint := ini.ReadString(FormName, (C as TBitBtn).Name + 'Hint', (C as TBitBtn).Hint);
          continue;
        end;
        if c.ClassName = 'TOpenDialog' then
        begin
          (C as TOpenDialog).Title := ini.ReadString(FormName, (C as TOpenDialog).Name + 'Title', (C as TOpenDialog).Title);
          (C as TOpenDialog).Filter := ini.ReadString(FormName, (C as TOpenDialog).Name + 'Filter', (C as TOpenDialog).Filter);
          continue;
        end;
        if c.ClassName = 'TSaveDialog' then
        begin
          (C as TSaveDialog).Title := ini.ReadString(FormName, (C as TSaveDialog).Name + 'Title', (C as TSaveDialog).Title);
          (C as TSaveDialog).Filter := ini.ReadString(FormName, (C as TSaveDialog).Name + 'Filter', (C as TSaveDialog).Filter);
          continue;
        end;
        if c.ClassName = 'TToolButton' then
        begin
          (C as TToolButton).Caption := ini.ReadString(FormName, (C as TToolButton).Name, (C as TToolButton).Caption);
          (C as TToolButton).Hint := ini.ReadString(FormName, (C as TToolButton).Name + 'Hint', (C as TToolButton).Hint);
          continue;
        end;
        if c.ClassName = 'TSpeedButton' then
        begin
          (C as TSpeedButton).Caption := ini.ReadString(FormName, (C as TSpeedButton).Name, (C as TSpeedButton).Caption);
          (C as TSpeedButton).Hint := ini.ReadString(FormName, (C as TSpeedButton).Name + 'Hint', (C as TSpeedButton).Hint);
          continue;
        end;
        if c.ClassName = 'TRadioButton' then
        begin
          (C as TRadioButton).Caption := ini.ReadString(FormName, (C as TRadioButton).Name, (C as TRadioButton).Caption);
          (C as TRadioButton).Hint := ini.ReadString(FormName, (C as TRadioButton).Name + 'Hint', (C as TRadioButton).Hint);
          continue;
        end;
        if c.ClassName = 'TMenuItem' then
        begin
          s := (C as TMenuItem).Caption;
          (C as TMenuItem).Caption := ini.ReadString(FormName, (C as TMenuItem).Name, (C as TMenuItem).Caption);
          (C as TMenuItem).Hint := ini.ReadString(FormName, (C as TMenuItem).Name + 'Hint', (C as TMenuItem).Hint);
          continue;
        end;
        if c.ClassName = 'TComboBox' then
        begin
          ItemCount := (c as TComboBox).Items.Count;
          ItemString := ini.ReadString(FormName, (C as TComboBox).Name + 'Items', (c as TComboBox).Items.Text);
          (c as TComboBox).Items.Text := StringReplace(ItemString, ';', #13#10, [rfReplaceAll]);
          (C as TComboBox).Hint := ini.ReadString(FormName, (C as TComboBox).Name + 'Hint', (C as TComboBox).Hint);
          continue;
        end;
          if c.ClassName = 'TListBox' then
          begin
          ItemCount := (c as TListBox).Items.Count;
          ItemString := ini.ReadString(FormName, (C as TListBox).Name + 'Items', (c as TListBox).ITems.Text);
          (c as TListBox).Items.Text := StringReplace(ItemString, ';', #13#10, [rfReplaceAll]);
          (C as TListBox).Hint := ini.ReadString(FormName, (C as TListBox).Name + 'Hint', (C as TListBox).Hint);
          continue;
        end;
      end;
    finally
      ini.Free;
    end;
  end;
end;

procedure TDFUseLng.ReplaceLF(var value: string; ReplaceWith: string);
var
  i: Integer;
  vPos: Integer;
begin
  vPos := Pos(#13#10, value);
  while vPos > 0 do
  begin
    Delete(value, vPos, 2);
    Insert(ReplaceWith, value, vPos);
    vPos := Pos(#13#10, value);
  end;
  if value <> '' then
    if value[Length(value)] = ';' then
      Delete(value, Length(value), 1);
end;

function TDFUseLng.SetActiveLanguage(RootKey: dword; Key, value,
  Lan: string): boolean;
var
  vReg: TRegistry;
begin
  Result := false;
  vReg := TRegistry.Create;
  try
    vReg.RootKey := RootKey;

    if vReg.OpenKey(Key, true) then
    begin
      vReg.WriteString(value, Lan);
      vReg.CloseKey();
    end;
  finally
    vReg.Free;
  end;
end;

procedure TDFUseLng.SetLanguageFilesDirectory(value: string);
begin
  if FLanguageFilesDirectory <> value then
  begin
    FLanguageFilesDirectory := value;
    if FLanguageFilesDirectory[Length(FLanguageFilesDirectory)] = '\' then
      Delete(FLanguageFilesDirectory, Length(FLanguageFilesDirectory), 1);
  end;
end;

end.
